/** 
	A program in Qt
	that use OpenGL drawing primitives
	File: 4cubes.cpp

	Fraga 28/11/2000
**/
#include <qpainter.h>
#include <stdio.h>
#include <GL/glu.h>
#include "cube.h"

Cube::Cube( QWidget *parent, const char *name )
		: QGLWidget( parent, name )
{
	signo = 1;	

	angle = 0.0;
	// setMouseTracking( TRUE );

	setFocusPolicy(QWidget::StrongFocus);

	timer = new QTimer( this, "mytimer" );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	for(int i=0; i<16; i++)
	if(i%5!=0) m[i]=0.0;
	else m[i]=1.0;

	val = 0;
	fovy = 94.0;
	far = 0.0;
	flag = 0;
}


void Cube::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Cube::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

//
// This method puts the initial coordinates each time window is resized
//
void Cube::resizeGL( int w, int h )
{
	aspect=(double)w/(double)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if ( w <= h ) { 
		// glOrtho ( -1.0, 1.0, -1.0/aspect, 1.0/aspect, -2.0, 2.0);
		gluPerspective( fovy, 1.0/aspect, 0.1, 20.0 );
		aspect = 1.0/aspect;
	}
	else {
		// glOrtho ( -aspect, aspect, -1.0, 1.0, -2.0, 2.0);
		gluPerspective( fovy, aspect, 0.1, 20.0 );
	}

	emit changeSize ( );
}

QSizePolicy Cube::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

/** 
	Function that generate a cube with center in (cx,cy,cz)
	with size value for its side
**/
void Cube::getcube ( float cx, float cy, float cz, float size, float v[8][3] ) 
{
	v[0][0] = v[3][0] = v[4][0] = v[7][0] = cx;
	v[0][1] = v[1][1] = v[2][1] = v[3][1] = cy;
	v[0][2] = v[1][2] = v[4][2] = v[5][2] = cz;
	v[1][0] = v[2][0] = v[5][0] = v[6][0] = cx + size;
	v[4][1] = v[5][1] = v[6][1] = v[7][1] = cy + size;
	v[2][2] = v[3][2] = v[6][2] = v[7][2] = cz + size;
}

void Cube::show_cube ( float v[8][3] )
{
	static GLfloat n[6][3] = {
		{ 0.0, 0.0, 1.0},
		{ 1.0, 0.0, 0.0},
		{ 0.0, 0.0,-1.0},
		{-1.0, 0.0, 0.0},
		{ 0.0, 1.0, 0.0},
		{ 0.0,-1.0, 0.0} };

	static GLint faces[6][4] = {
		{7,6,2,3},
		{6,5,1,2},
		{5,1,0,4},
		{4,0,3,7},
		{4,5,6,7},
		{0,1,2,3} };

	static GLfloat colors[6][3] = {
		{ 1.0, 1.0, 1.0}, // white
		{ 1.0, 0.0, 0.0}, // red
		{ 0.0, 1.0, 0.0}, // green
		{ 0.0, 0.0, 1.0}, // blue
		{ 1.0, 1.0, 0.0}, // yellow
		{ 1.0, 0.0, 1.0}  // magenta 
	};

	int i;

	for (i = 5; i >= 0; i--) {
		glBegin(GL_QUADS);
		// glBegin(GL_LINE_LOOP);
		glColor3fv( &colors[i][0] );
		glNormal3fv(&n[i][0]);
		glVertex3fv(&v[faces[i][0]][0]);
		glVertex3fv(&v[faces[i][1]][0]);
		glVertex3fv(&v[faces[i][2]][0]);
		glVertex3fv(&v[faces[i][3]][0]);
		glEnd();
	}
}

void Cube::paintGL(void)
{
	#define SINGLEINS  0
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if ( flag ) {
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective( fovy, aspect, 0.1, 20.0 );
		flag=0;
	}
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */
	
	glTranslatef( 0.0, 0.0, far );
	glPushMatrix( );
	// gluLookAt( 0.0, 0.0, far, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 ); 
	glMultMatrixf( m );
	glRotatef (xrot, 1.0, 0.0, 0.0);
	glRotatef (yrot, 0.0, 1.0, 0.0);

	// glScalef ( 0.7, 0.7, 0.7);	  /* modeling transformation */ 

	glPushMatrix( );
#ifdef SINGLEINS
	glRotatef ( angle,  1.0, 1.0, 1.0);
#else
	glRotatef ( 45.0,  0.0, 1.0, 0.0);
	glRotatef (-35.26, 1.0, 0.0, 0.0);
	glRotatef ( angle, 0.0, 0.0, 1.0);
	glRotatef ( 35.26, 1.0, 0.0, 0.0);
	glRotatef (-45.0,  0.0, 1.0, 0.0);
#endif
	show_cube( c1 );

	glPopMatrix( );
	glPopMatrix( );

	glBegin(GL_LINES);
		// Draw x axis:
		glColor3f( 1.0, 0.0, 0.0 );
		glVertex3f(-0.1, 0.0, 0.0);
		glVertex3f( 1.1, 0.0, 0.0);
		// Draw y axis:
		glVertex3f( 0.0, -0.1, 0.0);
		glVertex3f( 0.0,  1.1, 0.0);
		// Draw z axis:
		glVertex3f( 0.0, 0.0, -0.1);
		glVertex3f( 0.0, 0.0,  1.1);
	glEnd();
	
	swapBuffers();
}

void Cube::initializeGL( ) 
{
	glClearColor (0.8, 0.8, 0.8, 0.0);	// Background to white

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);
	gluPerspective( fovy, 1.0, 0.1, 20.0 );

	// getcube ( 0.0, 0.0, 0.0, 1.0, c1 );
	getcube (-0.5,-0.5,-0.5, 1.0, c1 );

	glEnable (GL_DEPTH_TEST);
}

void Cube::keyPressEvent( QKeyEvent *e )
{
	switch ( e->key() ) {
		case Key_Left:
			fovy -= 1;	
			flag=1;
			updateGL( );
			printf("fovy = %d\n", (int)fovy );
			break;
		case Key_Right:
			fovy += 1;	
			flag=1;
			updateGL( );
			printf("fovy = %d\n", (int)fovy );
			break;
		case Key_Up:
			far += 0.1;	
			updateGL( );
			printf("%1.1f\n", far );
			break;
		case Key_Down:
			far -= 0.1;	
			printf("%1.1f\n", far );
			updateGL( );
			break;
		case Key_R:
			xrot = yrot = angle = 0.0;	
			updateGL( );
			break;
		case Key_Plus:
			val++;
			m[8]=(float)val*0.1; 
			m[9] = m[8];
			updateGL( );
			break;
		case Key_Minus:
			val--;
			m[8]=(float)val*0.1; 
			m[9] = m[8];
			updateGL( );
			break;
		default:
			return;
	}
	e->accept( );
}


